// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
// all rights reserved.
// licensed to the public under the terms of the GNU GPL (>= 2)
// see the file COPYING for details

#include <stdio.h>
#include <stdarg.h>
#include <iostream>
#include <string>

#include <boost/lexical_cast.hpp>

#include "sanity.hh"

using namespace std;

// debugging / logging system

sanity global_sanity;

sanity::sanity() : verbose(false), logbuf(NULL)
{
  logbuf = L_buffer_create(0xffff);
  I(logbuf);
  L_buffer_wraparound(logbuf, 1);
}

sanity::~sanity()
{
  if (logbuf)
    L_buffer_delete(logbuf);
}

void sanity::set_verbose()
{
  verbose = true;
}

void sanity_printf(sanity & s, const char *format, ...)
{
  int const sz = 4096;
  va_list ap;
  va_start(ap, format);
  char buf[sz];
  vsnprintf(buf, sz, format, ap);
  va_end(ap);
  L_buffer_puts(s.logbuf, buf);
  if (s.verbose)
    {
      cerr << buf;
      cerr.flush();
    }
}

void sanity::dump_buffer()
{
  if (logbuf)
    L_buffer_dump(logbuf, stdout);
}

void sanity_invariant_handler(char *expr, char *file, int line, int dummy)
{
  if (global_sanity.logbuf)
    L("invariant '%s' violated at %s:%d\n", expr, file, line);
  else
    cerr << "invariant '" << expr << "' violated at " << file << ":" << line << endl;
  throw std::logic_error(string(file) + string(":")
			 + boost::lexical_cast<string>(line) + string(": ")
			 + string(expr));
}

