#ifndef __SANITY_HH__
#define __SANITY_HH__

// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
// all rights reserved.
// licensed to the public under the terms of the GNU GPL (>= 2)
// see the file COPYING for details

#include <stdexcept>
#include <L_buffer.h>

// our assertion / sanity / error logging system is based on GNU Nana with
// some minor customizations (namely: an assert failure throws, and the
// logging is done to core)

struct sanity {
  sanity();
  ~sanity();
  void dump_buffer();
  void set_verbose();
  bool verbose;
  L_BUFFER * logbuf;
};

typedef std::runtime_error oops;

extern sanity global_sanity;
void sanity_invariant_handler(char *expr, char *file, int line, int dummy);

#ifdef __GNUC__
void sanity_printf(sanity & s, const char *format, ...)
  __attribute__((format (printf, 2, 3)));
#else
void sanity_printf(sanity & s, const char *format, ...);
#endif

#define I_DEFAULT_HANDLER sanity_invariant_handler
#define I_DEFAULT_PARAMS 0
#define L_DEFAULT_HANDLER sanity_printf
#define L_DEFAULT_PARAMS global_sanity

#include <nana.h>

#endif // __SANITY_HH__
